import { createDynamicScopeExpression } from '../src/dynamic-scope'

test('Complete dynamic scope implementation in non-strict mode', () => {
  function named() {
    return message
  }

  const scope = createDynamicScopeExpression(named)
  const message = 'Hello World!'
  const func = eval(scope)()

  expect(func()).toEqual('Hello World!')
})

test('Incomplete dynamic scope implementation in strict mode', () => {
  'use strict'

  function named() {
    return message
  }

  const scope = createDynamicScopeExpression(named)
  const message = 'Hello World!'
  const func = eval(scope)()

  expect(() => func()).toThrow(ReferenceError)
})

test('Closure variables non-strict', () => {
  const fn = whoami => () => whoami

  const scope = createDynamicScopeExpression(fn('Lisa'))
  const func = eval(scope)()

  expect(() => func()).toThrow(ReferenceError)
})

test('Closure variables strict', () => {
  'use strict'
  const fn = whoami => () => whoami

  const scope = createDynamicScopeExpression(fn('Lisa'))
  const func = eval(scope)()

  expect(() => func()).toThrow(ReferenceError)
})

test('Nesting non-strict', () => {
  function say() {
    return text
  }

  const scope1 = createDynamicScopeExpression(say)
  const scope2 = createDynamicScopeExpression(eval(scope1)())
  const text = 'Hi'
  const func = eval(scope2)()

  expect(func()).toEqual('Hi')
})

test('Nesting strict', () => {
  'use strict'

  function say() {
    return text
  }

  const scope1 = createDynamicScopeExpression(say)
  const scope2 = createDynamicScopeExpression(eval(scope1)())
  const func = eval(scope2)({ text: 'Hi' })

  expect(func()).toEqual('Hi')
})